==> thx_organization-0.1.0-py3-none-any.whl: thx/organization/models/abstracts.py <==
# -*- coding: utf-8 -*-
import base64
import hashlib
import os

import swapper
from django.conf import settings
from django.core.files import File
from django.db import models
from django.utils import timezone
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _
from thx.carriage.models import Delivery
from thx.core.models import CleanModel, DateModel, OrderedModel, StatusModel, UserModel
from thx.currency.models import Currency
from thx.i18n.models import Language
from thx.office.models import Office
from thx.payment.models import PaymentForm
from thx.vat.models import Vat


def get_upload_avatar_path(instance, filename):
    today_path = f"{timezone.now():%Y/%m/%d/}"
    first_name = slugify(instance.first_name)
    last_name = slugify(instance.last_name)
    name_path = last_name[:3]
    name, ext = os.path.splitext(filename)
    filename = '{}_{}_avatar{}'.format(last_name, first_name, ext)
    return os.path.join("customer", "avatar", today_path, name_path, filename)


class AbstractOrganization(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    code = models.CharField(
        max_length=50,
        verbose_name=_("code"),
    )
    office = models.ForeignKey(
        Office,
        on_delete=models.PROTECT,
        related_name="%(class)ss",
        blank=True, null=True,
        verbose_name=_("office"),
    )
    organization_type = models.ForeignKey(
        "organization.OrganizationType",
        on_delete=models.PROTECT,
        related_name="%(class)ss",
        blank=True, null=True,
        verbose_name=_("organization type"),
    )
    organization_form = models.ForeignKey(
        "organization.OrganizationForm",
        on_delete=models.PROTECT,
        related_name="%(class)ss",
        blank=True, null=True,
        verbose_name=_("organization form"),
    )
    payment_form = models.ForeignKey(
        PaymentForm,
        on_delete=models.PROTECT,
        related_name="%(class)ss",
        blank=True, null=True,
        verbose_name=_('payment form'),
    )
    currency = models.ForeignKey(
        Currency,
        on_delete=models.PROTECT,
        related_name="%(class)ss",
        blank=True, null=True,
        verbose_name=_("currency"),
    )
    delivery = models.ForeignKey(
        Delivery,
        on_delete=models.PROTECT,
        related_name="%(class)ss",
        blank=True, null=True,
        verbose_name=_("delivery"),
    )
    vat = models.ForeignKey(
        Vat,
        on_delete=models.PROTECT,
        related_name="%(class)ss",
        blank=True, null=True,
        verbose_name=_("vat"),
    )
    name = models.CharField(
        max_length=255,
        db_index=True,
        verbose_name=_("name"),
    )
    brand = models.CharField(
        max_length=255,
        blank=True,
        db_index=True,
        verbose_name=_("brand"),
    )
    ssn = models.CharField(
        max_length=16,
        blank=True, null=True,
        db_index=True,
        verbose_name=_("ssn"),
    )
    vat_number = models.CharField(
        max_length=14,
        blank=True, null=True,
        db_index=True,
        verbose_name=_("vat number"),
    )
    sdi = models.CharField(
        max_length=7,
        blank=True, null=True,
        verbose_name=_("SDI"),
    )
    url = models.URLField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_("url"),
    )
    email = models.EmailField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_("email"),
    )
    phone = models.CharField(
        max_length=20,
        blank=True, null=True,
        verbose_name=_("phone"),
    )
    phone2 = models.CharField(
        blank=True, null=True,
        max_length=20,
        verbose_name=_("phone2"),
    )
    fax = models.CharField(
        blank=True, null=True,
        max_length=20,
        verbose_name=_("fax"),
    )
    note = models.TextField(
        blank=True,
        verbose_name=_("note"),
    )

    class Meta:
        abstract = True
        ordering = ('ordering', 'name',)

    def __str__(self):
        return self.name

    def clean(self):
        self.set_organization_style()
        super().clean()

    def set_organization_style(self):
        osn = settings.ORGANIZATION_ORGANIZATION_NAME_STYLE
        if osn == "title":
            self.name = self.name.title()
        elif osn == "lower":
            self.name = self.name.lower()
        elif osn == "upper":
            self.name = self.name.upper()

    def get_addresses(self):
        """
        Return all active addresses
        """
        return self.addresses.filter(status=1)

    def get_first_address(self):
        """
        Return first active address
        """
        addresses = self.get_addresses()
        if addresses:
            return self.get_addresses().first()
        return None

    def get_shipping_addresses(self):
        """
        Return a list of active shipping addresses
        """
        return self.get_addresses().filter(
            address_scopes__is_shipping_address=True
        )

    def get_invoice_addresses(self):
        """
        Return a list of active invoice addresses
        """
        return self.get_addresses().filter(
            address_scopes__is_invoice_address=True)

    @property
    def category(self):
        if self.organization_type:
            return self.organization_type.get_category_display()
        return None

    @property
    def default_invoice_address1(self):
        """
        Return first active invoice address.
        address1 = `address.type + address.name + address.number`
        """
        addresses = self.get_invoice_addresses()
        if addresses:
            address = addresses.first()
            return "{address} {name}, {number}".format(
                address=address.get_address_type_display() or "",
                name=address.name,
                number=address.number,
            )
        return None

    @property
    def default_invoice_address2(self):
        """
        Returns first active invoice address.
        address2 = `address.zip_code + address.locality + address.province`
        """
        addresses = self.get_invoice_addresses()
        if addresses:
            address = addresses.first()
            return "{zipcode} {locality} ({province})".format(
                zipcode=address.zip_code,
                locality=address.locality,
                province=address.province,
            )
        return None

    @property
    def default_invoice_address3(self):
        """
        Returns first active invoice address.
        address3 = `address.country`
        """
        addresses = self.get_invoice_addresses()
        if addresses:
            address = addresses.first()
            return "{country}".format(
                country=address.country,
            )
        return None

    @property
    def default_shipping_address1(self):
        """
        Returns first active shipping address.
        address1 = `address.type + address.name + address.number`
        """
        addresses = self.get_shipping_addresses()
        if addresses:
            address = addresses.first()
            return "{address} {name}, {number}".format(
                address=address.get_address_type_display() or "",
                name=address.name,
                number=address.number,
            )
        return None

    @property
    def default_shipping_address2(self):
        """
        Returns first active shipping address.
        address2 = `address.zip_code + address.locality + address.province`
        """
        addresses = self.get_shipping_addresses()
        if addresses:
            address = addresses.first()
            return "{zipcode} {locality} ({province})".format(
                zipcode=address.zip_code,
                locality=address.locality,
                province=address.province,
            )
        return None

    @property
    def default_shipping_address3(self):
        """
        Returns first active shipping address.
        address3 = `address.country`
        """
        addresses = self.get_shipping_addresses()
        if addresses:
            address = addresses.first()
            return "{country}".format(
                country=address.country,
            )
        return None

    @property
    def office_name(self):
        return self.office.name if self.office else None

    @property
    def organization_type_name(self):
        return self.organization_type.name if self.organization_type else None

    @property
    def organization_form_name(self):
        return self.organization_form.name if self.organization_form else None

    @property
    def currency_name(self):
        return self.currency.name if self.currency else None

    @property
    def delivery_transport_by(self):
        return self.delivery.transport_by if self.delivery else None

    @property
    def vat_name(self):
        return self.vat.name if self.vat else None


class AbstractContact(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    code = models.CharField(
        max_length=50,
        unique=True,
        verbose_name=_("code"),
    )
    user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        on_delete=models.PROTECT,
        related_name='%(class)s',
        blank=True, null=True,
        verbose_name=_("user"),
    )
    last_name = models.CharField(
        max_length=255,
        verbose_name=_('last name'),
    )
    first_name = models.CharField(
        max_length=255,
        verbose_name=_('first name'),
    )
    email = models.EmailField(
        verbose_name=_('email'),
    )
    mistakes = models.PositiveIntegerField(
        default=0,
        verbose_name=_('mistakes'),
    )
    agree_privacy = models.BooleanField(
        default=False,
        verbose_name=_('agree privacy'),
    )
    agree_terms = models.BooleanField(
        default=False,
        verbose_name=_('agree terms'),
    )
    agree_advertising = models.BooleanField(
        default=False,
        verbose_name=_('agree advertising material'),
    )
    avatar = models.ImageField(
        _("avatar"),
        blank=True, null=True,
        upload_to=get_upload_avatar_path,
    )
    mail_profile_not_completed_sent = models.BooleanField(
        default=False,
        verbose_name=_('mail profile not completed sent'),
    )

    # superuser contacts
    is_superuser = models.BooleanField(
        default=False,
        verbose_name=_('is superuser'),
    )
    # staff contacts
    is_staff = models.BooleanField(
        default=False,
        verbose_name=_('is staff'),
    )
    organization = models.ForeignKey(
        swapper.get_model_name('organization', 'Organization'),
        on_delete=models.PROTECT,
        related_name='contacts',
        blank=True, null=True,
        verbose_name=_("organization"),
    )
    title = models.ForeignKey(
        "organization.ContactTitle",
        on_delete=models.PROTECT,
        related_name='contacts',
        blank=True, null=True,
        verbose_name=_("title"),
    )
    roles = models.ManyToManyField(
        "organization.ContactRole",
        related_name='contacts',
        blank=True,
        verbose_name=_("roles"),
    )
    language = models.ForeignKey(
        Language,
        on_delete=models.PROTECT,
        related_name='contacts',
        blank=True, null=True,
        verbose_name=_('language'),
    )
    note = models.TextField(
        blank=True,
        verbose_name=_("note"),
    )

    class Meta:
        abstract = True
        verbose_name = _('contact')
        verbose_name_plural = _('contacts')
        ordering = ('ordering', 'user__last_name', 'user__first_name')
        permissions = (
            ("list_contact", "Can list contact"),
            ("detail_contact", "Can detail contact"),
            ("enable_contact", "Can enable contact"),
            ("disable_contact", "Can disable contact"),
        )

    def __str__(self):
        return "{last_name} {first_name}".format(
            last_name=self.last_name,
            first_name=self.first_name,
        )

    def save(self, *args, **kwargs):
        self.set_code()
        super().save(*args, **kwargs)

    def clean(self):
        self.set_contact_style()
        self.set_disable_for_mistakes()
        super().clean()

    def set_code(self):
        if not self.code:
            time = timezone.now().strftime('%Y%m%d%H%M%S%f')
            code = hashlib.md5(time.encode('utf-8')).hexdigest()
            self.code = code

    def set_contact_style(self):
        osn = settings.ORGANIZATION_CONTACT_NAME_STYLE
        if osn == "title":
            self.last_name = self.last_name.title()
            self.first_name = self.first_name.title()
        elif osn == "lower":
            self.last_name = self.last_name.lower()
            self.first_name = self.first_name.lower()
        elif osn == "upper":
            self.last_name = self.last_name.upper()
            self.first_name = self.first_name.upper()

    def set_disable_for_mistakes(self):
        if self.mistakes >= settings.ORGANIZATION_CONTACT_MISTAKES_MAX:
            self.user.is_active = False
            self.user.save()

    @property
    def organization_name(self):
        return self.organization.name if self.organization else None

    @property
    def full_name(self):
        """
        returns the contact's full name (last name and first name)
        """
        return " ".join((self.last_name.strip(), self.first_name.strip())).strip()

    @property
    def full_name_initial(self):
        """
        returns the contact's initial (last name and first name)
        """
        return ".".join([name[0] for name in self.full_name.split(" ")]) + "."

    @property
    def signature(self):
        """
        returns the contact's full name (first name and last name)
        """
        return " ".join((self.first_name, self.last_name)).strip()

    @property
    def signature_initial(self):
        """
        returns the contact's signature initial (first name and last name)
        """
        return "".join([name[0] for name in self.signature.split(" ")])

    @property
    def user_email(self):
        return self.user.email if self.user else None

    @property
    def username(self):
        return self.user.username if self.user else None

    @property
    def title_name(self):
        return self.title.name if self.title else None

    @property
    def get_role_list(self):
        return [role for role in self.roles.all()]

    @property
    def language_name(self):
        return self.language.name if self.language else None

    @property
    def avatar_base64(self):
        if self.avatar and hasattr(self.avatar, 'path'):
            try:
                with open(self.avatar.path, 'rb') as f:
                    image = File(f)
                    data = base64.b64encode(image.read())
                return data.decode('utf-8')
            except FileNotFoundError:
                return None
        return None

    @classmethod
    def subscribe(cls, data, user, **kwargs):
        email = data.get('email')
        first_name = data.get('first_name')
        last_name = data.get('last_name')
        agree_privacy = data.get('agree_privacy')
        agree_terms = data.get('agree_terms')
        agree_advertising = data.get('agree_advertising')
        contact = cls(
            creator=user, last_modifier=user,
            email=email, first_name=first_name, last_name=last_name, user=user,
            agree_privacy=agree_privacy, agree_terms=agree_terms, agree_advertising=agree_advertising,
            **kwargs
        )
        contact.save()
        return contact

==> thx_organization-0.1.1-py3-none-any.whl: thx/organization/models/abstracts.py <==
# -*- coding: utf-8 -*-
import base64
import hashlib
import os

import swapper
from django.conf import settings
from django.core.files import File
from django.db import models
from django.utils import timezone
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _
from thx.carriage.models import Delivery
from thx.core.models import CleanModel, DateModel, OrderedModel, StatusModel, UserModel
from thx.currency.models import Currency
from thx.i18n.models import Language
from thx.office.models import Office
from thx.payment.models import PaymentForm
from thx.vat.models import Vat


def get_upload_avatar_path(instance, filename):
    today_path = f"{timezone.now():%Y/%m/%d/}"
    first_name = slugify(instance.first_name)
    last_name = slugify(instance.last_name)
    name_path = last_name[:3]
    name, ext = os.path.splitext(filename)
    filename = '{}_{}_avatar{}'.format(last_name, first_name, ext)
    return os.path.join("customer", "avatar", today_path, name_path, filename)


class AbstractOrganization(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    code = models.CharField(
        max_length=50,
        verbose_name=_("code"),
    )
    office = models.ForeignKey(
        Office,
        on_delete=models.PROTECT,
        related_name="%(class)ss",
        blank=True, null=True,
        verbose_name=_("office"),
    )
    organization_type = models.ForeignKey(
        "organization.OrganizationType",
        on_delete=models.PROTECT,
        related_name="%(class)ss",
        blank=True, null=True,
        verbose_name=_("organization type"),
    )
    organization_form = models.ForeignKey(
        "organization.OrganizationForm",
        on_delete=models.PROTECT,
        related_name="%(class)ss",
        blank=True, null=True,
        verbose_name=_("organization form"),
    )
    payment_form = models.ForeignKey(
        PaymentForm,
        on_delete=models.PROTECT,
        related_name="%(class)ss",
        blank=True, null=True,
        verbose_name=_('payment form'),
    )
    currency = models.ForeignKey(
        Currency,
        on_delete=models.PROTECT,
        related_name="%(class)ss",
        blank=True, null=True,
        verbose_name=_("currency"),
    )
    delivery = models.ForeignKey(
        Delivery,
        on_delete=models.PROTECT,
        related_name="%(class)ss",
        blank=True, null=True,
        verbose_name=_("delivery"),
    )
    vat = models.ForeignKey(
        Vat,
        on_delete=models.PROTECT,
        related_name="%(class)ss",
        blank=True, null=True,
        verbose_name=_("vat"),
    )
    name = models.CharField(
        max_length=255,
        db_index=True,
        verbose_name=_("name"),
    )
    brand = models.CharField(
        max_length=255,
        blank=True,
        db_index=True,
        verbose_name=_("brand"),
    )
    ssn = models.CharField(
        max_length=16,
        blank=True, null=True,
        db_index=True,
        verbose_name=_("ssn"),
    )
    vat_number = models.CharField(
        max_length=14,
        blank=True, null=True,
        db_index=True,
        verbose_name=_("vat number"),
    )
    sdi = models.CharField(
        max_length=7,
        blank=True, null=True,
        verbose_name=_("SDI"),
    )
    url = models.URLField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_("url"),
    )
    email = models.EmailField(
        max_length=100,
        blank=True, null=True,
        verbose_name=_("email"),
    )
    phone = models.CharField(
        max_length=20,
        blank=True, null=True,
        verbose_name=_("phone"),
    )
    phone2 = models.CharField(
        blank=True, null=True,
        max_length=20,
        verbose_name=_("phone2"),
    )
    fax = models.CharField(
        blank=True, null=True,
        max_length=20,
        verbose_name=_("fax"),
    )
    note = models.TextField(
        blank=True,
        verbose_name=_("note"),
    )

    class Meta:
        abstract = True
        ordering = ('ordering', 'name',)

    def __str__(self):
        return self.name

    def clean(self):
        self.set_organization_style()
        super().clean()

    def set_organization_style(self):
        if self.name is not None:
            osn = settings.ORGANIZATION_ORGANIZATION_NAME_STYLE
            if osn == "title":
                self.name = self.name.title()
            elif osn == "lower":
                self.name = self.name.lower()
            elif osn == "upper":
                self.name = self.name.upper()

    def get_addresses(self):
        """
        Return all active addresses
        """
        return self.addresses.filter(status=1)

    def get_first_address(self):
        """
        Return first active address
        """
        addresses = self.get_addresses()
        if addresses:
            return self.get_addresses().first()
        return None

    def get_shipping_addresses(self):
        """
        Return a list of active shipping addresses
        """
        return self.get_addresses().filter(
            address_scopes__is_shipping_address=True
        )

    def get_invoice_addresses(self):
        """
        Return a list of active invoice addresses
        """
        return self.get_addresses().filter(
            address_scopes__is_invoice_address=True
        )

    @property
    def category(self):
        if self.organization_type:
            return self.organization_type.get_category_display()
        return None

    @property
    def default_invoice_address1(self):
        """
        Return first active invoice address.
        address1 = `address.type + address.name + address.number`
        """
        addresses = self.get_invoice_addresses()
        if addresses:
            address = addresses.first()
            return "{address} {name}, {number}".format(
                address=address.get_address_type_display() or "",
                name=address.name,
                number=address.number,
            )
        return None

    @property
    def default_invoice_address2(self):
        """
        Returns first active invoice address.
        address2 = `address.zip_code + address.locality + address.province`
        """
        addresses = self.get_invoice_addresses()
        if addresses:
            address = addresses.first()
            return "{zipcode} {locality} ({province})".format(
                zipcode=address.zip_code,
                locality=address.locality,
                province=address.province,
            )
        return None

    @property
    def default_invoice_address3(self):
        """
        Returns first active invoice address.
        address3 = `address.country`
        """
        addresses = self.get_invoice_addresses()
        if addresses:
            address = addresses.first()
            return "{country}".format(
                country=address.country,
            )
        return None

    @property
    def default_shipping_address1(self):
        """
        Returns first active shipping address.
        address1 = `address.type + address.name + address.number`
        """
        addresses = self.get_shipping_addresses()
        if addresses:
            address = addresses.first()
            return "{address} {name}, {number}".format(
                address=address.get_address_type_display() or "",
                name=address.name,
                number=address.number,
            )
        return None

    @property
    def default_shipping_address2(self):
        """
        Returns first active shipping address.
        address2 = `address.zip_code + address.locality + address.province`
        """
        addresses = self.get_shipping_addresses()
        if addresses:
            address = addresses.first()
            return "{zipcode} {locality} ({province})".format(
                zipcode=address.zip_code,
                locality=address.locality,
                province=address.province,
            )
        return None

    @property
    def default_shipping_address3(self):
        """
        Returns first active shipping address.
        address3 = `address.country`
        """
        addresses = self.get_shipping_addresses()
        if addresses:
            address = addresses.first()
            return "{country}".format(
                country=address.country,
            )
        return None

    @property
    def office_name(self):
        return self.office.name if self.office else None

    @property
    def organization_type_name(self):
        return self.organization_type.name if self.organization_type else None

    @property
    def organization_form_name(self):
        return self.organization_form.name if self.organization_form else None

    @property
    def currency_name(self):
        return self.currency.name if self.currency else None

    @property
    def delivery_transport_by(self):
        return self.delivery.transport_by if self.delivery else None

    @property
    def vat_name(self):
        return self.vat.name if self.vat else None


class AbstractContact(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    code = models.CharField(
        max_length=50,
        unique=True,
        verbose_name=_("code"),
    )
    user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        on_delete=models.PROTECT,
        related_name='%(class)s',
        blank=True, null=True,
        verbose_name=_("user"),
    )
    last_name = models.CharField(
        max_length=255,
        verbose_name=_('last name'),
    )
    first_name = models.CharField(
        max_length=255,
        verbose_name=_('first name'),
    )
    email = models.EmailField(
        verbose_name=_('email'),
    )
    mistakes = models.PositiveIntegerField(
        default=0,
        verbose_name=_('mistakes'),
    )
    agree_privacy = models.BooleanField(
        default=False,
        verbose_name=_('agree privacy'),
    )
    agree_terms = models.BooleanField(
        default=False,
        verbose_name=_('agree terms'),
    )
    agree_advertising = models.BooleanField(
        default=False,
        verbose_name=_('agree advertising material'),
    )
    avatar = models.ImageField(
        _("avatar"),
        blank=True, null=True,
        upload_to=get_upload_avatar_path,
    )
    mail_profile_not_completed_sent = models.BooleanField(
        default=False,
        verbose_name=_('mail profile not completed sent'),
    )

    # superuser contacts
    is_superuser = models.BooleanField(
        default=False,
        verbose_name=_('is superuser'),
    )
    # staff contacts
    is_staff = models.BooleanField(
        default=False,
        verbose_name=_('is staff'),
    )
    organization = models.ForeignKey(
        swapper.get_model_name('organization', 'Organization'),
        on_delete=models.PROTECT,
        related_name='contacts',
        blank=True, null=True,
        verbose_name=_("organization"),
    )
    title = models.ForeignKey(
        "organization.ContactTitle",
        on_delete=models.PROTECT,
        related_name='contacts',
        blank=True, null=True,
        verbose_name=_("title"),
    )
    roles = models.ManyToManyField(
        "organization.ContactRole",
        related_name='contacts',
        blank=True,
        verbose_name=_("roles"),
    )
    language = models.ForeignKey(
        Language,
        on_delete=models.PROTECT,
        related_name='contacts',
        blank=True, null=True,
        verbose_name=_('language'),
    )
    note = models.TextField(
        blank=True,
        verbose_name=_("note"),
    )

    class Meta:
        abstract = True
        verbose_name = _('contact')
        verbose_name_plural = _('contacts')
        ordering = ('ordering', 'user__last_name', 'user__first_name')
        permissions = (
            ("list_contact", "Can list contact"),
            ("detail_contact", "Can detail contact"),
            ("enable_contact", "Can enable contact"),
            ("disable_contact", "Can disable contact"),
        )

    def __str__(self):
        return "{last_name} {first_name}".format(
            last_name=self.last_name,
            first_name=self.first_name,
        )

    def save(self, *args, **kwargs):
        self.set_code()
        super().save(*args, **kwargs)

    def clean(self):
        self.set_contact_style()
        self.set_disable_for_mistakes()
        super().clean()

    def set_code(self):
        if not self.code:
            time = timezone.now().strftime('%Y%m%d%H%M%S%f')
            code = hashlib.md5(time.encode('utf-8')).hexdigest()
            self.code = code

    def set_contact_style(self):
        osn = settings.ORGANIZATION_CONTACT_NAME_STYLE
        if self.last_name is not None:
            if osn == "title":
                self.last_name = self.last_name.title()
            elif osn == "lower":
                self.last_name = self.last_name.lower()
            elif osn == "upper":
                self.last_name = self.last_name.upper()
        if self.first_name is not None:
            if osn == "title":
                self.first_name = self.first_name.title()
            elif osn == "lower":
                self.first_name = self.first_name.lower()
            elif osn == "upper":
                self.first_name = self.first_name.upper()

    def set_disable_for_mistakes(self):
        if self.mistakes >= settings.ORGANIZATION_CONTACT_MISTAKES_MAX:
            self.user.is_active = False
            self.user.save()

    @property
    def organization_name(self):
        return self.organization.name if self.organization else None

    @property
    def full_name(self):
        """
        returns the contact's full name (last name and first name)
        """
        return " ".join((self.last_name.strip(), self.first_name.strip())).strip()

    @property
    def full_name_initial(self):
        """
        returns the contact's initial (last name and first name)
        """
        return ".".join([name[0] for name in self.full_name.split(" ")]) + "."

    @property
    def signature(self):
        """
        returns the contact's full name (first name and last name)
        """
        return " ".join((self.first_name, self.last_name)).strip()

    @property
    def signature_initial(self):
        """
        returns the contact's signature initial (first name and last name)
        """
        return "".join([name[0] for name in self.signature.split(" ")])

    @property
    def user_email(self):
        return self.user.email if self.user else None

    @property
    def username(self):
        return self.user.username if self.user else None

    @property
    def title_name(self):
        return self.title.name if self.title else None

    @property
    def get_role_list(self):
        return [role for role in self.roles.all()]

    @property
    def language_name(self):
        return self.language.name if self.language else None

    @property
    def avatar_base64(self):
        if self.avatar and hasattr(self.avatar, 'path'):
            try:
                with open(self.avatar.path, 'rb') as f:
                    image = File(f)
                    data = base64.b64encode(image.read())
                return data.decode('utf-8')
            except FileNotFoundError:
                return None
        return None

    @classmethod
    def subscribe(cls, data, user, **kwargs):
        email = data.get('email')
        first_name = data.get('first_name')
        last_name = data.get('last_name')
        agree_privacy = data.get('agree_privacy')
        agree_terms = data.get('agree_terms')
        agree_advertising = data.get('agree_advertising')
        contact = cls(
            creator=user, last_modifier=user,
            email=email, first_name=first_name, last_name=last_name, user=user,
            agree_privacy=agree_privacy, agree_terms=agree_terms, agree_advertising=agree_advertising,
            **kwargs
        )
        contact.save()
        return contact
